@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #!/usr/bin/env ruby
 
 require 'test/unit'
@@ -14,13 +15,13 @@ class TestParser < Test::Unit::TestCase
 
   # Test creating a new element 
   def test_new_element 
-    elem = Hpricot::Elem.new(Hpricot::STag.new('form')) 
+    elem = Hpricot::Elem.new('form') 
     assert_not_nil(elem) 
     assert_not_nil(elem.attributes) 
   end 
 
   def test_scan_text
-    assert_equal 'FOO', Hpricot.make("FOO").first.content
+    assert_equal 'FOO', Hpricot.make("FOO").children.first.content
   end
 
   def test_filter_by_attr
@@ -47,6 +48,13 @@ class TestParser < Test::Unit::TestCase
     assert_equal 'link1', @basic.get_elements_by_tag_name('a')[0].get_attribute('id')
     assert_equal 'link1', @basic.get_elements_by_tag_name('body')[0].get_element_by_id('link1').get_attribute('id')
   end
+  
+  def test_get_elements_by_tag_name_star
+    simple = Hpricot.parse("<div><p id='first'>First</p><p id='second'>Second</p></div>")
+    assert_equal 3, simple.get_elements_by_tag_name("*").size
+    assert_equal 1, simple.get_elements_by_tag_name("div").size
+    assert_equal 2, simple.get_elements_by_tag_name("p").size
+  end
 
   def test_output_basic
     @basic = Hpricot.parse(TestFiles::BASIC)
@@ -114,7 +122,7 @@ class TestParser < Test::Unit::TestCase
     assert_equal 60, @boingboing.search("h3").length
     assert_equal 59, @boingboing.search("h3[text()!='College kids reportedly taking more smart drugs']").length
     assert_equal 17, @boingboing.search("h3[text()$='s']").length
-    assert_equal 129, @boingboing.search("p[text()]").length
+    assert_equal 116, @boingboing.search("p[text()]").length
     assert_equal 211, @boingboing.search("p").length
   end
 
@@ -154,10 +162,10 @@ class TestParser < Test::Unit::TestCase
     assert_equal 60, @boingboing.search("/*/body//p[@class='posted']").length
     assert_equal 18, @boingboing.search("//script").length
     divs = @boingboing.search("//script/../div")
-    assert_equal 1,  divs.length
+    assert_equal 2,  divs.length
     imgs = @boingboing.search('//div/p/a/img')
-    assert_equal 15, imgs.length
-    assert_equal 17, @boingboing.search('//div').search('p/a/img').length
+    assert_equal 16, imgs.length
+    assert_equal 16, @boingboing.search('//div').search('p/a/img').length
     assert imgs.all? { |x| x.name == 'img' }
   end
 
@@ -165,10 +173,10 @@ class TestParser < Test::Unit::TestCase
     @boingboing = Hpricot.parse(TestFiles::BOINGBOING)
     assert_equal 2, @boingboing.search('//link[@rel="alternate"]').length
     p_imgs = @boingboing.search('//div/p[/a/img]')
-    assert_equal 15, p_imgs.length
+    assert_equal 16, p_imgs.length
     assert p_imgs.all? { |x| x.name == 'p' }
     p_imgs = @boingboing.search('//div/p[a/img]')
-    assert_equal 18, p_imgs.length
+    assert_equal 16, p_imgs.length
     assert p_imgs.all? { |x| x.name == 'p' }
     assert_equal 1, @boingboing.search('//input[@checked]').length
   end
@@ -211,7 +219,7 @@ class TestParser < Test::Unit::TestCase
   def test_many_paths
     @boingboing = Hpricot.parse(TestFiles::BOINGBOING)
     assert_equal 62, @boingboing.search('p.posted, link[@rel="alternate"]').length
-    assert_equal 20, @boingboing.search('//div/p[a/img]|//link[@rel="alternate"]').length
+    assert_equal 18, @boingboing.search('//div/p[a/img]|//link[@rel="alternate"]').length
   end
 
   def test_stacked_search
@@ -219,6 +227,14 @@ class TestParser < Test::Unit::TestCase
     assert_kind_of Hpricot::Elements, @boingboing.search('//div/p').search('a img')
   end
 
+  def test_attr_casing
+    doc = Hpricot("<a HREF='a'>A simple <b>test</b> string.</a>")
+    assert_equal (doc % :a)[:href], "a"
+    assert_equal (doc % :a)[:HREF], nil
+    assert_equal (doc % :a)['href'], "a"
+    assert_equal (doc % :a)['HREF'], nil
+  end
+
   def test_class_search
     # test case sent by Chih-Chao Lam
     doc = Hpricot("<div class=xyz'>abc</div>")
@@ -299,10 +315,8 @@ class TestParser < Test::Unit::TestCase
     assert_equal "blah='blah'", doc.children[2].content
   end
 
-  def test_buffer_error
-    assert_raise Hpricot::ParseError, "ran out of buffer space on element <input>, starting on line 3." do
-      Hpricot(%{<p>\n\n<input type="hidden" name="__VIEWSTATE"  value="#{(("X" * 2000) + "\n") * 22}" />\n\n</p>})
-    end
+  def test_no_buffer_error
+    Hpricot(%{<p>\n\n<input type="hidden" name="__VIEWSTATE"  value="#{(("X" * 2000) + "\n") * 44}" />\n\n</p>})
   end
 
   def test_youtube_attr
@@ -376,4 +390,39 @@ class TestParser < Test::Unit::TestCase
     assert (doc/"//t:sam").size > 0 # at least this should probably work
     # assert (doc/"//sam").size > 0  # this would be nice 
   end
+
+  def test_uxs_ignores_non_entities
+    assert_equal 'abc', Hpricot.uxs('abc')
+  end
+
+  def test_uxs_handles_gt_lt_amp_quot
+    assert_equal '"&<>', Hpricot.uxs('&quot;&amp;&lt;&gt;')
+  end
+
+  def test_uxs_handles_numeric_values
+    if String.method_defined? :encoding
+      assert_equal "é", Hpricot.uxs('&#233;')
+    else
+      assert_equal "\303\251", Hpricot.uxs('&#233;')
+    end
+  end
+
+  def test_uxs_handles_entities
+    if String.method_defined? :encoding
+      assert_equal "é", Hpricot.uxs('&eacute;')
+    else
+      assert_equal "\303\251", Hpricot.uxs('&eacute;')
+    end
+  end
+
+  def test_cdata_inner_text
+    xml = Hpricot.XML(%{
+      <peon>
+        <id>96586</id>
+        <stdout><![CDATA[This is STDOUT]]></stdout>
+        <stderr><!-- IGNORE --><![CDATA[This is]]> STDERR</stderr>
+      </peon>})
+      assert_equal "This is STDOUT", (xml/:peon/:stdout).inner_text
+      assert_equal "This is STDERR", (xml/:peon/:stderr).inner_text
+  end
 end
